// jshint esversion: 6
	import * as THREE from 'three';
	import {FontLoader} from 'three/addons/loaders/FontLoader.js';
	import {TextGeometry} from 'three/addons/geometries/TextGeometry.js';

	// base mod
	export function klystron(text='KSP', myscale=0.625) {
		const width=1500, height=1500, depth=1500;
		const ModK = new THREE.Object3D();
		const ModContainer = new THREE.Object3D();
		const materialBase = new THREE.MeshLambertMaterial({color: 0xff0000});
		const klystron = new THREE.Object3D();
		let group, textMesh, textGeo, materials;
		let size = 60;
		let textheight = 10;
		let pos = [];
		let rot = [];
		const hover = 30;
		const curveSegments = 8;
		let font;
		function insertText(isize, ipos, irot) {
			group = new THREE.Group();
			group.position.y = 100;
			klystron.add( group );
			size = isize;
			pos = ipos;
			rot = irot;
			loadFont();
		}
		function loadFont() {
			const loader = new FontLoader();
			// https://threejs.org/examples/fonts/droid_sans_bold.typeface.json
			loader.load( './fonts/droid_sans_bold.typeface.json', function ( response ) {
				font = response;
				refreshText();
			} );
		}
		materials = [
			new THREE.MeshPhongMaterial( { color: 0x333333, flatShading: true } ), // front
			new THREE.MeshPhongMaterial( { color: 0x333333 } ) // side
		];
		function createText() {
			textGeo = new TextGeometry(text.replace('TKLY_KG','K').replace('.01',''), {font: font, size: size, height: textheight, curveSegments: curveSegments});
			textGeo.computeBoundingBox();
			const centerOffset = - 0.5 * ( textGeo.boundingBox.max.x - textGeo.boundingBox.min.x );
			textMesh = new THREE.Mesh( textGeo, materials );
			textMesh.position.x = pos[0];
			textMesh.position.y = pos[1];
			textMesh.position.z = pos[2];
			textMesh.rotation.x = rot[0];
			textMesh.rotation.y = rot[1];
			textMesh.rotation.z = rot[2];
			group.add( textMesh );
		}
		function refreshText() {
			group.remove( textMesh );
			if ( ! text ) return;
			createText();
		}
		insertText(75, [950,680,560], [0, Math.PI / 2, 0]);
		
		const basegeometry = new THREE.BoxGeometry(width, height, depth);
		const d1mesh = new THREE.Mesh(basegeometry, materialBase);
		ModContainer.add(d1mesh);

		// cover
		const d2geometry = new THREE.BoxGeometry(width+100, 100, depth+100);
		const dmaterial2= new THREE.MeshLambertMaterial({color: 0xf0f0f0});
		const d2mesh = new THREE.Mesh(d2geometry, dmaterial2);
		d2mesh.position.set(0, 800, 0);		
		ModContainer.add(d2mesh);	
		
  		// base
		const d3geometry = new THREE.BoxGeometry(width+100, 100, depth+100);
		const d3mesh = new THREE.Mesh(d3geometry, materialBase);
		d3mesh.position.set(0, -800, 0);		
		ModContainer.add(d3mesh);	
	
		// big cylinder
		const d4geometry = new THREE.CylinderGeometry(400, 400, 1200, 50);
		const textureLoader = new THREE.TextureLoader();
		const texture = textureLoader.load('./components/mod_texture.png');	
		texture.wrapS = texture.wrapT = THREE.RepeatWrapping;
		texture.repeat.set(5, 5);		
		const material = new THREE.MeshBasicMaterial({ map: texture });		
		const d4mesh = new THREE.Mesh(d4geometry, material);
		d4mesh.rotateX(Math.PI * 1);
		d4mesh.rotateY(Math.PI * 1);
		d4mesh.rotateZ(Math.PI * 1);
		d4mesh.position.set(0, 970 , 300);			
		ModContainer.add(d4mesh);	

		// small cylinder
		const d5geometry = new THREE.CylinderGeometry(200, 200, 1200, 50);
		const textureLoader2 = new THREE.TextureLoader();
		const texture2 = textureLoader.load('./components/mod_texture.png');	
		texture2.wrapS = texture2.wrapT = THREE.RepeatWrapping;
		texture2.repeat.set(3, 3);		
		const material2 = new THREE.MeshBasicMaterial({ map: texture2 });		
		const d5mesh = new THREE.Mesh(d5geometry, material2);
		d5mesh.rotateX(Math.PI * 1);
		d5mesh.rotateY(Math.PI * 1);
		d5mesh.rotateZ(Math.PI * 1);
		d5mesh.position.set(0, 1670 , 300);			
		ModContainer.add(d5mesh);	
		
		// high cover
		const d6geometry = new THREE.CylinderGeometry(220, 220, 90, 50);
		const materialGreyDark= new THREE.MeshLambertMaterial({color: 0x9d9d9d});
		const d6mesh = new THREE.Mesh(d6geometry, materialGreyDark);
		d6mesh.rotateX(Math.PI * 1);
		d6mesh.rotateY(Math.PI * 1);
		d6mesh.rotateZ(Math.PI * 1);
		d6mesh.position.set(0, 2300 , 300);			
		ModContainer.add(d6mesh);	
		
		// right arm
		const Bracciodx = new THREE.BoxGeometry( 600, 100, 200 );
		const d7mesh = new THREE.Mesh(Bracciodx, materialBase);
		d7mesh.rotateX(Math.PI * 1);
		d7mesh.rotateY(Math.PI * 1.25);
		d7mesh.rotateZ(Math.PI * 1);		
		d7mesh.position.set(300, 1300 , 600);			
		ModContainer.add(d7mesh);	

		// left arm
		const Bracciosx = new THREE.BoxGeometry( 600, 100, 200 );
		const d8mesh = new THREE.Mesh(Bracciosx, materialBase);
		d8mesh.rotateX(Math.PI * 1);
		d8mesh.rotateY(Math.PI * 1.75);
		d8mesh.rotateZ(Math.PI * 1);		
		d8mesh.position.set(300, 1300 , 0);			
		ModContainer.add(d8mesh);	

		// right arm continue
		const PBracciodx = new THREE.BoxGeometry( 600, 100, 190 );
		const d9mesh = new THREE.Mesh(PBracciodx, materialBase);
		d9mesh.rotateX(Math.PI * 1);
		d9mesh.rotateY(Math.PI * 1);
		d9mesh.rotateZ(Math.PI * 1);		
		d9mesh.position.set(743, 1300 , 790);			
		ModContainer.add(d9mesh);			
		
		//prolung braccia sx
		const PBracciosx = new THREE.BoxGeometry( 600, 100, 190 );
		const d10mesh = new THREE.Mesh(PBracciosx, materialBase);
		d10mesh.rotateX(Math.PI * 1);
		d10mesh.rotateY(Math.PI * 1);
		d10mesh.rotateZ(Math.PI * 1);		
		d10mesh.position.set(743, 1300 , -190);			
		ModContainer.add(d10mesh);	
		
		//trave 1 base
		const trave1 = new THREE.BoxGeometry( 80, 1500, 190 );
		const materialtrave= new THREE.MeshLambertMaterial({color: 0xd80000});
		const d12mesh = new THREE.Mesh(trave1, materialtrave);
		d12mesh.rotateX(Math.PI * 1);
		d12mesh.rotateY(Math.PI * 1);
		d12mesh.rotateZ(Math.PI * 1);		
		d12mesh.position.set(-760, 0 , -300);			
		ModContainer.add(d12mesh);	
		
		//trave 2 base
		const trave2 = new THREE.BoxGeometry( 80, 1500, 190 );
		const d13mesh = new THREE.Mesh(trave1, materialtrave);
		d13mesh.rotateX(Math.PI * 1);
		d13mesh.rotateY(Math.PI * 1);
		d13mesh.rotateZ(Math.PI * 1);		
		d13mesh.position.set(-760, 0 , 300);			
		ModContainer.add(d13mesh);	

		//targhetta
		const targhetta = new THREE.BoxGeometry( 10, 300, 300 );
		const d14mesh = new THREE.Mesh(targhetta, dmaterial2);
		d14mesh.position.set(-750, 550 , 0);
		ModContainer.add(d14mesh);	
		
		//Torus basso sx
		const torus_bs = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d15mesh = new THREE.Mesh( torus_bs, materialGreyDark );
		d15mesh.rotateX(Math.PI * 1);
		d15mesh.rotateY(Math.PI * 1.5);
		d15mesh.rotateZ(Math.PI * 1);	
		d15mesh.position.set(-700, 900 , -600);
		ModContainer.add(d15mesh);	
	
		//Torus basso dx
		const torus_bd = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d16mesh = new THREE.Mesh( torus_bd, materialGreyDark );
		d16mesh.rotateX(Math.PI * 1);
		d16mesh.rotateY(Math.PI * 1.5);
		d16mesh.rotateZ(Math.PI * 1);	
		d16mesh.position.set(-700, 900 , 600);
		ModContainer.add(d16mesh);	
	
		//Torus alto sx
		const torus_as = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d17mesh = new THREE.Mesh( torus_as, materialGreyDark );
		d17mesh.rotateX(Math.PI * 1);
		d17mesh.rotateY(Math.PI * 1.5);
		d17mesh.rotateZ(Math.PI * 1);	
		d17mesh.position.set(700, 900 , -600);
		ModContainer.add(d17mesh);	

		//Torus alto dx
		const torus_ad = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d18mesh = new THREE.Mesh( torus_ad, materialGreyDark );
		d18mesh.rotateX(Math.PI * 1);
		d18mesh.rotateY(Math.PI * 1.5);
		d18mesh.rotateZ(Math.PI * 1);	
		d18mesh.position.set(700, 900 , 600);
		ModContainer.add(d18mesh);
		ModContainer.rotateY(Math.PI);
		ModContainer.scale.set(myscale, myscale, myscale);
		ModContainer.position.set(width*myscale/2, height*myscale/2, depth*myscale/2);
		klystron.add(ModContainer);
		return klystron;
	}
